# hint in each org report
# either tells you "nothing in this world" or important item is here
# random, but always hints to proof locations
# randomly picks a template of one of the reports from one of the games
# tells you the world or the world and how (chest/etc)