from dataclasses import dataclass, field

from List.configDict import itemType
from List.inventory.item import InventoryItem


@dataclass(frozen=True)
class Map(InventoryItem):
    id: int
    name: str
    type: itemType = field(init=False, default=itemType.MAP)


CastleMap = Map(90, "Castle Map")
BasementMap = Map(91, "Basement Map")
CastleWallsMap = Map(92, "Castle Walls Map")
EncampmentAreaMap = Map(112, "Encampment Area Map")
VillageAreaMap = Map(113, "Village Area Map")
CornerstoneHillMap = Map(114, "Cornerstone Hill Map")
LilliputMap = Map(116, "Lilliput Map")
BuildingSiteMap = Map(117, "Building Site Map")
MickeysHouseMap = Map(118, "Mickey's House Map")
DisneyCastleMap = Map(119, "Disney Castle Map")
AgrabahMap = Map(120, "Agrabah Map")
CaveOfWondersMap = Map(121, "Cave of Wonders Map")
RuinsMap = Map(122, "Ruins Map")
UnderseaKingdomMap = Map(123, "Undersea Kingdom Map")
StarryHillMap = Map(124, "Starry Hill Map")
HundredAcreWoodMap = Map(125, "100 Acre Wood Map")
RabbitsHowseMap = Map(126, "Rabbit's House Map")
PigletsHowseMap = Map(127, "Piglet's House Map")
KangasHowseMap = Map(128, "Kanga's House Map")
SpookyCaveMap = Map(129, "Spooky Cave Map")
PalaceMap = Map(130, "Palace Map")
ColiseumMap = Map(134, "Coliseum Map")
UnderworldMap = Map(135, "Underworld Map")
CavernsMap = Map(136, "Caverns Map")
HalloweenTownMap = Map(250, "Halloween Town Map")
NavalMap = Map(251, "Naval Map")
PrideRockMap = Map(252, "Pride Rock Map")
MarketplaceMap = Map(253, "Marketplace Map")
PitCellAreaMap = Map(254, "Pit Cell Area Map")
TwilightTownMap = Map(255, "Twilight Town Map")
DarkCityMap = Map(256, "Dark City Map")
InterceptorMap = Map(505, "The Interceptor Map")
BlackPearlMap = Map(506, "The Black Pearl Map")
IslaDeMuertaMap = Map(507, "Isla de Muerta Map")
ShipGraveyardMap = Map(508, "Ship Graveyard Map")
ChristmasTownMap = Map(509, "Christmas Town Map")
CurlyHillMap = Map(510, "Curly Hill Map")
OasisMap = Map(511, "Oasis Map")
SavannahMap = Map(512, "Savannah Map")
CastlePerimeterMap = Map(513, "Castle Perimeter Map")
GreatMawMap = Map(514, "The Great Maw Map")
IoTowerMap = Map(515, "I/O Tower Map")
CentralComputerCoreMap = Map(516, "Central Computer Core Map")
SolarSailerSimulationMap = Map(517, "Solar Sailer Simulation Map")
WindowOfTimeMap = Map(518, "Window of Time Map")
SunsetHillMap = Map(531, "Sunset Hill Map")
MansionMap = Map(532, "Mansion Map")
TowerMap = Map(533, "Tower Map")
CastleThatNeverWasMap = Map(536, "Castle That Never Was Map")
DarkRemembranceMap = Map(586, "Dark Remembrance Map")
DepthsOfRemembranceMap = Map(590, "Depths of Remembrance Map")
GardenOfAssemblageMap = Map(592, "Garden of Assemblage Map")
