ABILITY_POOL = "ability_pool"
ACCESSIBILITY = "seed_accessibility"
ACCESSORIES_IN_ITEM_POOL = "accessories_in_item_pool"
ALLOW_PROOF_HINTING = "allow_proof_hinting"
ALLOW_REPORT_HINTING = "allow_report_hinting"
ANTIFORM = "antiform"
ARMOR_IN_ITEM_POOL = "armor_in_item_pool"
AS_DATA_SPLIT = "as_data_reward_split"
ATLANTICA_TUTORIAL_SKIP = "skip_atlantica_tutorial"
AUTO_EQUIP_START_ABILITIES = "auto_equip"
BATTLE_LEVEL_OFFSET = "battle_level_offset"
BATTLE_LEVEL_RANDO = "battle_level_rando"
BATTLE_LEVEL_RANDOM_MAX = "battle_level_random_max"
BATTLE_LEVEL_RANDOM_MIN = "battle_level_random_min"
BATTLE_LEVEL_RANGE = "battle_level_range"
BLOCK_COR_SKIP = "block_cor_skip"
BLOCK_SHAN_YU_SKIP = "block_shan_yu_skip"
CHAIN_LOGIC = "chain_logic"
CHAIN_LOGIC_LENGTH = "chain_logic_length"
MAX_CHAIN_LOGIC_LENGTH = "max_chain_logic_length"
CHAIN_LOGIC_MIN_TERRA = "chain_logic_min_terra"
CHAIN_LOGIC_TERRA = "chain_logic_terra"
CHESTS_MATCH_ITEM = "chests_match_item"
COMMAND_MENU = "command_menu"
COSTUME_RANDO = "costume_rando"
CRITICAL_BONUS_REWARDS = "critical_bonus_rewards"
CUPS_GIVE_XP = "cups_give_xp"
DISABLE_FINAL_FORM = "disable_final_form_behavior"
DONALD_AP = "donald_ap"
DOUBLE_STAT_GROWTH_MIN_LEVEL = "double_stat_growth_min_level"
DOUBLE_STAT_GROWTH_MAX_LEVEL = "double_stat_growth_max_level"
EMBLEM_NUM_REQUIRED = "emblem_num_required"
EMBLEM_NUM_AVAILABLE = "emblem_num_available"
ENABLE_PROMISE_CHARM = "enable_promise_charm"
ENDPIC_RANDO = "endpic_rando"
FAST_URNS = "fast_urns"
FIFTY_AP_BOOSTS = "fifty_ap_boosts"
FINAL_DOOR_REQUIREMENT = "final_door_requirement"
FINAL_EXP_CURVE = "final_exp_curve"
FINAL_EXP_MULTIPLIER = "final_exp_multiplier"
GARDEN_OF_ASSEMBLAGE_REWARDS = "garden_of_assemblage_rewards"
GLASS_CANNON = "glass_cannon"
GLOBAL_JACKPOT = "global_jackpot"
GLOBAL_LUCKY = "global_lucky"
GOOFY_AP = "goofy_ap"
HINT_SYSTEM = "hint_system"
HINTABLE_CHECKS = "hintable_checks"
ITEM_PLACEMENT_DIFFICULTY = "item_placement_difficulty"
ITEMPIC_RANDO = "itempic_rando"
JUNK_ITEMS = "junk_items"
KEYBLADE_ACTION_ABILITIES = "keyblade_action_abilities"
KEYBLADE_SUPPORT_ABILITIES = "keyblade_support_abilities"
KEYBLADE_MAX_STAT = "keyblade_max_stat"
KEYBLADE_MIN_STAT = "keyblade_min_stat"
KEYBLADE_RANDO = "keyblade_rando"
KEYBLADE_RANDO_ALLOW_DUPLICATES = "keyblade_rando_allow_duplicates"
KEYBLADE_RANDO_INCLUDE_EFFECTS = "keyblade_rando_include_effects"
KEYBLADES_LOCK_CHESTS = "keyblades_lock_chests"
LIMIT_EXP_CURVE = "limit_exp_curve"
LIMIT_EXP_MULTIPLIER = "limit_exp_multiplier"
MAPS_IN_ITEM_POOL = "maps_in_item_pool"
MASTER_EXP_CURVE = "master_exp_curve"
MASTER_EXP_MULTIPLIER = "master_exp_multiplier"
MISC_LOCATIONS_WITH_REWARDS = "misc_locations_with_rewards"
MUNNY_IN_ITEM_POOL = "munny_in_item_pool"
MUSIC_RANDO_ENABLED_PC = "music_rando_enabled_pc"
MUSIC_RANDO_PC_ALLOW_DUPLICATES = "music_rando_pc_allow_duplicates"
MUSIC_RANDO_PC_DMCA_SAFE = "music_rando_pc_dmca_safe"
MUSIC_RANDO_PC_INCLUDE_BBS = "music_rando_pc_include_all_bbs"
MUSIC_RANDO_PC_INCLUDE_CUSTOM = "music_rando_pc_include_custom"
MUSIC_RANDO_PC_INCLUDE_DDD = "music_rando_pc_include_all_ddd"
MUSIC_RANDO_PC_INCLUDE_KH1 = "music_rando_pc_include_all_kh1"
MUSIC_RANDO_PC_INCLUDE_KH2 = "music_rando_pc_include_all_kh2"
MUSIC_RANDO_PC_INCLUDE_RECOM = "music_rando_pc_include_all_recom"
MUSIC_RANDO_PC_USE_CATEGORIES = "music_rando_pc_use_categories"
NIGHTMARE_LOGIC = "nightmare_logic"
OBJECTIVE_RANDO_NUM_REQUIRED = "objective_rando_num_required"
OBJECTIVE_RANDO_NUM_AVAILABLE = "objective_rando_num_available"
OBJECTIVE_RANDO_POOL = "objective_rando_pool"
PARTY_MEMBER_RANDO = "party_member_rando"
POINTS_ABILITY = "points_abilities"
POINTS_ABILITY_COLLECT = "points_ability_collect"
POINTS_AUX = "points_aux"
POINTS_BONUS = "points_bonus"
POINTS_BOSS_AS = "points_boss_as"
POINTS_BOSS_DATA = "points_boss_data"
POINTS_BOSS_FINAL = "points_boss_final"
POINTS_BOSS_NORMAL = "points_boss_normal"
POINTS_BOSS_SEPHIROTH = "points_boss_sephiroth"
POINTS_BOSS_TERRA = "points_boss_terra"
POINTS_COMPLETE = "points_complete"
POINTS_DEATH = "points_death"
POINTS_FORM = "points_forms"
POINTS_FORM_COLLECT = "points_form_collect"
POINTS_FORMLV = "points_formlv"
POINTS_KEYBLADES = "points_keyblades"
POINTS_MAGIC = "points_magic"
POINTS_MAGIC_COLLECT = "points_magic_collect"
POINTS_PAGE = "points_page"
POINTS_PAGE_COLLECT = "points_page_collect"
POINTS_POUCHES_COLLECT = "points_pouches_collect"
POINTS_PROOF = "points_proofs"
POINTS_PROOF_COLLECT = "points_proof_collect"
POINTS_REPORT = "points_report"
POINTS_REPORT_COLLECT = "points_report_collect"
POINTS_SUMMON = "points_summon"
POINTS_SUMMON_COLLECT = "points_summon_collect"
POINTS_VISIT = "points_visit"
POINTS_VISIT_COLLECT = "points_visit_collect"
PR_MAP_SKIP = "pr_map_skip"
PREVENT_SELF_HINTING = "prevent_self_hinting"
PROGRESSION_HINTS = "progression_hints"
PROGRESSION_HINTS_COMPLETE_BONUS = "progression_complete_bonus"
PROGRESSION_HINTS_REPORT_BONUS = "progression_report_bonus"
PROGRESSION_HINTS_REVEAL_END = "progression_points_reveal_end"
PROGRESSION_POINT_SELECT = "progression_point_select"
PROMISE_CHARM_DEPTH = "promise_charm_depth"
PROOF_DEPTH = "proof_depth"
RANDO_THEMED_TEXTURES = "rando_themed_textures"
RANDOMIZED_SETTINGS = "randomized_settings"
RANDOMIZED_SETTINGS_ENABLED = "randomized_settings_enabled"
RECOLOR_TEXTURES = "recolor_textures"
RECOLOR_TEXTURES_COMPRESS = "recolor_textures_compress"
RECOLOR_TEXTURES_INCLUDE_EXTRAS = "recolor_textures_include_extras"
RECOLOR_TEXTURES_KEEP_CACHE = "recolor_textures_keep_cache"
RECIPES_IN_ITEM_POOL = "recipes_in_item_pool"
REMOVE_CUTSCENES = "remove_cutscenes"
REMOVE_DAMAGE_CAP = "remove_damage_cap"
REMOVE_POPUPS = "remove_popups"
REMOVE_WARDROBE_ANIMATION = "remove_wardrobe_animation"
REPORT_DEPTH = "report_depth"
REPORTS_REVEAL = "reports_reveal"
JOURNAL_HINTS_ABILITIES = "journal_hints_abilities"
RETRY_DARK_THORN = "retry_dark_thorn"
RETRY_DFX = "retry_data_final_xemnas"
REVEAL_COMPLETE = "reveal_complete"
REVENGE_LIMIT_RANDO = "revenge_limit_rando"
RICH_ENEMIES = "rich_enemies"
ROOM_TRANSITION_IMAGES = "room_transition_images"
ROXAS_ABILITIES_ENABLED = "roxas_abilities_enabled"
SCORE_MODE = "score_mode"
SHOP_BOOSTS = "shop_boosts"
SHOP_ELIXIRS = "shop_elixirs"
SHOP_KEYBLADES = "shop_keyblades"
SHOP_RECOVERIES = "shop_recoveries"
SHOP_REPORTS = "shop_reports"
SHOP_UNLOCKS = "shop_unlocks"
SKIP_CARPET_ESCAPE = "skip_carpet_escape"
SOFTLOCK_CHECKING = "softlock_checking"
SORA_AP = "sora_ap"
SORA_EXP_CURVE = "sora_exp_curve"
SORA_EXP_MULTIPLIER = "sora_exp_multiplier"
SORA_LEVELS = "sora_levels"
SORA_STR_RATE = "sora_strength_rate"
SORA_DEF_RATE = "sora_defense_rate"
SORA_MAG_RATE = "sora_magic_rate"
SORA_AP_RATE = "sora_ap_rate"
SPLIT_LEVELS = "split_levels"
SPOILER_REVEAL_TYPES = "spoiler_reveal_types"
STARTING_INVENTORY = "starting_inventory"
STARTING_MOVEMENT = "starting_movement"
STARTING_REPORTS = "starting_reports"
STARTING_MAGIC_FIRE = "starting_magic_fire"
STARTING_MAGIC_BLIZZARD = "starting_magic_blizzard"
STARTING_MAGIC_THUNDER = "starting_magic_thunder"
STARTING_MAGIC_CURE = "starting_magic_cure"
STARTING_MAGIC_MAGNET = "starting_magic_magnet"
STARTING_MAGIC_REFLECT = "starting_magic_reflect"
STARTING_KEYBLADES = "starting_keyblades"
STARTING_ABILITIES = "starting_abilities"
STARTING_PAGES = "starting_pages"
STARTING_DRIVES = "starting_drives"
STARTING_ITEMS = "starting_items"
STARTING_UNLOCKS_AG = "starting_unlocks_ag"
STARTING_UNLOCKS_BC = "starting_unlocks_bc"
STARTING_UNLOCKS_DC = "starting_unlocks_dc"
STARTING_UNLOCKS_HB = "starting_unlocks_hb"
STARTING_UNLOCKS_HT = "starting_unlocks_ht"
STARTING_UNLOCKS_LOD = "starting_unlocks_lod"
STARTING_UNLOCKS_OC = "starting_unlocks_oc"
STARTING_UNLOCKS_PL = "starting_unlocks_pl"
STARTING_UNLOCKS_PR = "starting_unlocks_pr"
STARTING_UNLOCKS_SP = "starting_unlocks_sp"
STARTING_UNLOCKS_STT = "starting_unlocks_stt"
STARTING_UNLOCKS_TT = "starting_unlocks_tt"
STARTING_UNLOCKS_TWTNW = "starting_unlocks_twtnw"
STARTING_VISIT_MODE = "starting_visit_mode"
STARTING_VISIT_RANDOM_MAX = "starting_visit_random_max"
STARTING_VISIT_RANDOM_MIN = "starting_visit_random_min"
STATSANITY = "statsanity"
STORY_UNLOCK_CATEGORY = "story_unlock_category"
STORY_UNLOCK_DEPTH = "story_unlock_depth"
SUMMON_EXP_CURVE = "summon_exp_curve"
SUMMON_EXP_MULTIPLIER = "summon_exp_multiplier"
SUPERBOSSES_WITH_REWARDS = "superbosses_with_rewards"
TEXTURE_RECOLOR_SETTINGS = "texture_recolor_settings"
UNLIMITED_MP = "unlimited_mp"
VALOR_EXP_CURVE = "valor_exp_curve"
VALOR_EXP_MULTIPLIER = "valor_exp_multiplier"
WEIGHTED_FORMS = "weighted_forms"
WEIGHTED_MAGIC = "weighted_magic"
WEIGHTED_PAGES = "weighted_pages"
WEIGHTED_PROMISE_CHARM = "weighted_promise_charm"
WEIGHTED_PROOFS = "weighted_proofs"
WEIGHTED_REPORTS = "weighted_reports"
WEIGHTED_SUMMONS = "weighted_summons"
WEIGHTED_UNLOCKS = "weighted_unlocks"
WISDOM_EXP_CURVE = "wisdom_exp_curve"
WISDOM_EXP_MULTIPLIER = "wisdom_exp_multiplier"
WORLDS_WITH_REWARDS = "worlds_with_rewards"
YEET_THE_BEAR = "yeet_the_bear"
DONALD_MELEE_ATTACKS_KNOCKBACK_TYPE = "donald_melee_attacks_knockback_type"
DONALD_FIRE_KNOCKBACK_TYPE = "donald_fire_knockback_type"
DONALD_BLIZZARD_KNOCKBACK_TYPE = "donald_blizzard_knockback_type"
DONALD_THUNDER_KNOCKBACK_TYPE = "donald_thunder_knockback_type"
GOOFY_MELEE_ATTACKS_KNOCKBACK_TYPE = "goofy_melee_attacks_knockback_type"
GOOFY_BASH_KNOCKBACK_TYPE = "goofy_bash_knockback_type"
GOOFY_TURBO_KNOCKBACK_TYPE = "goofy_turbo_knockback_type"
GOOFY_TORNADO_KNOCKBACK_TYPE = "goofy_tornado_knockback_type"
COMPANION_DAMAGE_TOGGLE = "companion_damage_toggle"
COMPANION_KILL_BOSS = "companion_kill_boss"
JACK_SKELLINGTON_MELEE_ATTACKS_KNOCKBACK_TYPE = "jack_skellington_melee_attacks_knockback_type"
SIMBA_MELEE_ATTACKS_KNOCKBACK_TYPE = "simba_melee_attacks_knockback_type"
ALADDIN_MELEE_ATTACKS_KNOCKBACK_TYPE = "aladdin_melee_attacks_knockback_type"
PING_MULAN_MELEE_ATTACKS_KNOCKBACK_TYPE = "ping_mulan_melee_attacks_knockback_type"
BEAST_MELEE_ATTACKS_KNOCKBACK_TYPE = "beast_melee_attacks_knockback_type"
TRON_MELEE_ATTACKS_KNOCKBACK_TYPE = "tron_melee_attacks_knockback_type"
JACK_SPARROW_MELEE_ATTACKS_KNOCKBACK_TYPE = "jack_sparrow_melee_attacks_knockback_type"
RIKU_MELEE_ATTACKS_KNOCKBACK_TYPE = "riku_melee_attacks_knockback_type"
AURON_MELEE_ATTACKS_KNOCKBACK_TYPE = "auron_melee_attacks_knockback_type"
RIKU_DARK_AURA_KNOCKBACK_TYPE = "riku_dark_aura_knockback_type"
RIKU_DARK_SHIELD_KNOCKBACK_TYPE = "riku_dark_shield_knockback_type"
AURON_DIVIDER_KNOCKBACK_TYPE = "auron_divider_knockback_type"
JACK_SPARROW_NO_MERCY_KNOCKBACK_TYPE = "jack_sparrow_no_mercy_knockback_type"
JACK_SPARROW_RAIN_STORM_KNOCKBACK_TYPE = "jack_sparrow_rain_time_knockback_type"
JACK_SPARROW_BONE_SMASH_KNOCKBACK_TYPE = "jack_sparrow_bone_smash_knockback_type"
TRON_SCOUTING_DISK_KNOCKBACK_TYPE = "tron_scouting_disk_knockback_type"
TRON_PULSING_THUNDER_KNOCKBACK_TYPE = "tron_pulsing_thunder_knockback_type"
JACK_SKELLINGTON_BLAZING_FURY_KNOCKBACK_TYPE = "jack_skellington_blazing_fury_knockback_type"
JACK_SKELLINGTON_ICY_TERROR_KNOCKBACK_TYPE = "jack_skellington_icy_terror_knockback_type"
JACK_SKELLINGTON_BOLTS_OF_SORROW_KNOCKBACK_TYPE = "jack_skellington_bolts_of_sorrow_knockback_type"
SIMBA_FIERCE_CLAW_KNOCKBACK_TYPE = "simba_fierce_claw_knockback_type"
SIMBA_GROUNDSHAKER_KNOCKBACK_TYPE = "simba_groundshaker_knockback_type"
ALADDIN_SLASH_FRENZY_KNOCKBACK_TYPE = "aladdin_slash_frenzy_knockback_type"
ALADDIN_QUICKPLAY_KNOCKBACK_TYPE = "aladdin_quickplay_knockback_type"
MULAN_MUSHU_FIRE_KNOCKBACK_TYPE = "mulan_mushu_fire_knockback_type"
MULAN_FLAMETONGUE_KNOCKBACK_TYPE = "mulan_flametongue_knockback_type"
BEAST_SHOUT_KNOCKBACK_TYPE = "beast_shout_knockback_type"
BEAST_RUSH_KNOCKBACK_TYPE = "beast_rush_knockback_type"
