kh2_music_list = [
    {
        "filename": "bgm\\music050.win32.scd",
        "title": "Dive into the Heart -Destati-",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music051.win32.scd",
        "title": "Fragments of Sorrow",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music052.win32.scd",
        "title": "The Afternoon Streets",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music053.win32.scd",
        "title": "Working Together",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music054.win32.scd",
        "title": "Sacred Moon",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music055.win32.scd",
        "title": "Deep Drive",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music059.win32.scd",
        "title": "A Fight to the Death",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music060.win32.scd",
        "title": "Darkness of the Unknown I",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music061.win32.scd",
        "title": "Darkness of the Unknown II",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music062.win32.scd",
        "title": "Darkness of the Unknown III",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music063.win32.scd",
        "title": "The 13th Reflection",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music064.win32.scd",
        "title": "What a Surprise?!",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music065.win32.scd",
        "title": "Happy Holidays!",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music066.win32.scd",
        "title": "The Other Promise",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music067.win32.scd",
        "title": "Rage Awakened",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music068.win32.scd",
        "title": "Cavern of Remembrance",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music069.win32.scd",
        "title": "Deep Anxiety",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music081.win32.scd",
        "title": "Beneath the Ground",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music082.win32.scd",
        "title": "The Escapade",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music084.win32.scd",
        "title": "Arabian Daydream",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music085.win32.scd",
        "title": "Byte Striking",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music086.win32.scd",
        "title": "Spooks of Halloween Town",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music087.win32.scd",
        "title": "Disappeared",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music088.win32.scd",
        "title": "Sora",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music089.win32.scd",
        "title": "Friends in My Heart",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music090.win32.scd",
        "title": "Riku",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music091.win32.scd",
        "title": "Kairi",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music092.win32.scd",
        "title": "A Walk in Andante",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music093.win32.scd",
        "title": "Villains of a Sort",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music094.win32.scd",
        "title": "Organization XIII",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music095.win32.scd",
        "title": "Apprehension",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music096.win32.scd",
        "title": "Courage",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music097.win32.scd",
        "title": "Laughter and Merriment",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music098.win32.scd",
        "title": "Hesitation",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music099.win32.scd",
        "title": "Missing You",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music100.win32.scd",
        "title": "The Underworld",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music101.win32.scd",
        "title": "Waltz of the Damned",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music102.win32.scd",
        "title": "What Lies Beneath",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music103.win32.scd",
        "title": "Olympus Coliseum",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music104.win32.scd",
        "title": "Dance of the Daring",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music106.win32.scd",
        "title": "Under the Sea (Japanese)",
        "type": ["Atlantica", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music107.win32.scd",
        "title": "Ursula's Revenge (Japanese)",
        "type": ["Atlantica", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music108.win32.scd",
        "title": "Part of Your World (Japanese)",
        "type": ["Atlantica", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music109.win32.scd",
        "title": "A New Day is Dawning (Japanese)",
        "type": ["Atlantica", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music110.win32.scd",
        "title": "The Encounter",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music111.win32.scd",
        "title": "Sinister Shadows",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music112.win32.scd",
        "title": "Fields of Honor",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music113.win32.scd",
        "title": "Swim This Way (Japanese)",
        "type": ["Atlantica", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music114.win32.scd",
        "title": "Tension Rising",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music115.win32.scd",
        "title": "The Corrupted",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music116.win32.scd",
        "title": "The Home of Dragons",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music117.win32.scd",
        "title": "Rowdy Rumble",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music118.win32.scd",
        "title": "Lazy Afternoons",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music119.win32.scd",
        "title": "Sinister Sundowns",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music120.win32.scd",
        "title": "Beneath the Ground",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music121.win32.scd",
        "title": "Desire for All That is Lost",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music122.win32.scd",
        "title": "Let's Sing and Dance! I",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music123.win32.scd",
        "title": "Let's Sing and Dance! II",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music127.win32.scd",
        "title": "A Day in Agrabah",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music128.win32.scd",
        "title": "Arabian Dream",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music129.win32.scd",
        "title": "Isn't It Lovely",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music130.win32.scd",
        "title": "Neverland Sky (KH I)",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music131.win32.scd",
        "title": "Dance to the Death",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music133.win32.scd",
        "title": "Magical Mystery",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music134.win32.scd",
        "title": "Working Together",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music135.win32.scd",
        "title": "Space Paranoids",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music136.win32.scd",
        "title": "Byte Bashing",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music137.win32.scd",
        "title": "A Twinkle in the Sky",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music138.win32.scd",
        "title": "Shipmeister's Shanty",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music139.win32.scd",
        "title": "Gearing Up",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music140.win32.scd",
        "title": "Under the Sea (KH I)",
        "type": ["Atlantica", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music141.win32.scd",
        "title": "Winnie the Pooh",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music142.win32.scd",
        "title": "Crossing the Finish Line",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music143.win32.scd",
        "title": "Mickey Mouse Club March",
        "type": ["Field", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music144.win32.scd",
        "title": "This is Halloween",
        "type": ["Field", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music145.win32.scd",
        "title": "Vim and Vigor",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music146.win32.scd",
        "title": "Roxas",
        "type": ["Cutscene", "Cutscene", "Wild"]
    },
    {
        "filename": "bgm\\music147.win32.scd",
        "title": "An Adventure in Atlantica (KH I)",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music148.win32.scd",
        "title": "Blast Off!",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music149.win32.scd",
        "title": "Spooks of Halloween Town",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music150.win32.scd",
        "title": "Squirming Evil (KH I)",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music151.win32.scd",
        "title": "The 13th Struggle",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music152.win32.scd",
        "title": "Reviving Hollow Bastion",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music153.win32.scd",
        "title": "Scherzo Di Notte",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music154.win32.scd",
        "title": "Nights of the Cursed",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music155.win32.scd",
        "title": "He's a Pirate (No Intro)",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music156.win32.scd",
        "title": "Guardando nel buio (KH I)",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music158.win32.scd",
        "title": "Bounce-O-Rama",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music159.win32.scd",
        "title": "Bounce-O-Rama (Speed Up Ver.)",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music164.win32.scd",
        "title": "Road to a Hero",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music185.win32.scd",
        "title": "The 13th Dilemma",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music186.win32.scd",
        "title": "Adventures in the Savannah",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music187.win32.scd",
        "title": "Savannah Pride",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music188.win32.scd",
        "title": "One-Winged Angel",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music189.win32.scd",
        "title": "Monochrome Dreams",
        "type": ["Field", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music190.win32.scd",
        "title": "Old Friends, Old Rivals",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music506.win32.scd",
        "title": "Under the Sea (English)",
        "type": ["Atlantica", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music507.win32.scd",
        "title": "Ursula's Revenge (English)",
        "type": ["Atlantica", "Battle", "Wild"]
    },
    {
        "filename": "bgm\\music508.win32.scd",
        "title": "Part of Your World (English)",
        "type": ["Atlantica", "Field", "Wild"],
        "dmca": True
    },
    {
        "filename": "bgm\\music509.win32.scd",
        "title": "A New Day is Dawning (English)",
        "type": ["Atlantica", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music513.win32.scd",
        "title": "Swim This Way (English)",
        "type": ["Atlantica", "Field", "Wild"]
    },
    {
        "filename": "bgm\\music517.win32.scd",
        "title": "Rowdy Rumble (Timeless River)",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music521.win32.scd",
        "title": "Desire for All That is Lost (Timeless River)",
        "type": ["Boss", "Boss", "Wild"]
    },
    {
        "filename": "bgm\\music530.win32.scd",
        "title": "He's a Pirate",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\End_Piano.win32.scd",
        "title": "Dearly Beloved -Reprise-",
        "type": ["Title", "Title", "Wild"]
    },
    {
        "filename": "vagstream\\GM1_Asteroid.win32.scd",
        "title": "Asteroid Attack",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\GM2_Highway.win32.scd",
        "title": "Hazardous Highway",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\GM3_Cloud.win32.scd",
        "title": "Cloudchasers",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\GM4_Floating.win32.scd",
        "title": "Floating in Bliss",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\GM5_Senkan.win32.scd",
        "title": "Battleship Bravery",
        "type": ["Battle", "Battle", "Wild"]
    },
    {
        "filename": "vagstream\\Title.win32.scd",
        "title": "Dearly Beloved",
        "type": ["Title", "Title", "Wild"]
    }
]

kh1_music_list = [
    # some music in kh1 is found in .dat folders and others in .bgm folders and somtimes both.
    # the ones in both .dat and .bgm folders are duplicates so you can ignore one of those.
    # DP
    {"name": "music101.dat\\music101.win32.scd", "kind": "field"},  # Deep Jungle
    {"name": "music102.dat\\music102.win32.scd", "kind": "battle"},  # Having a Wild Time
    {"name": "music157.bgm\\music157.win32.scd", "kind": "battle"},  # Holy Bananas
    # OC
    {"name": "music103.bgm\\music103.win32.scd", "kind": "field"},  # Olympus Coliseum
    {"name": "music164.bgm\\music164.win32.scd", "kind": "battle"},  # Road To Hero
    {"name": "music120.bgm\\music120.win32.scd", "kind": "battle"},  # Go For It
    # TT
    {"name": "music104.dat\\music104.win32.scd", "kind": "field"},  # Traverse Town
    {"name": "music112.dat\\music112.win32.scd", "kind": "battle"},  # Hand In Hand
    {"name": "music184.bgm\\music184.win32.scd", "kind": "field"},  # Merlin Magical House
    # DI
    {"name": "music105.dat\\music105.win32.scd", "kind": "field"},  # Destiny Islands
    {"name": "music119.dat\\music119.win32.scd", "kind": "battle"},  # Busting Up On The Beach
    # WL
    {"name": "music116.dat\\music116.win32.scd", "kind": "field"},  # Welcome To Wonderland
    {"name": "music118.bgm\\music118.win32.scd", "kind": "battle"},  # To Our Surprise
    # AG
    {"name": "music127.dat\\music127.win32.scd", "kind": "field"},  # A Day In Agrabah
    {"name": "music128.dat\\music128.win32.scd", "kind": "battle"},  # Arabian Dream
    # HT
    {"name": "music144.dat\\music144.win32.scd", "kind": "field", "dmca": True},  # This Is Halloween
    {"name": "music165.bgm\\music165.win32.scd", "kind": "field", "dmca": True},  # This Is Halloween (Alternative)
    {"name": "music149.dat\\music149.win32.scd", "kind": "battle"},  # Spooks Of Halloween Town
    # HB
    {"name": "music152.dat\\music152.win32.scd", "kind": "field"},  # Hollow Bastion
    {"name": "music153.dat\\music153.win32.scd", "kind": "battle"},  # Scherzo Di Notte
    # EW
    {"name": "music154.dat\\music154.win32.scd", "kind": "field"},  # End Of The World
    {"name": "music155.dat\\music155.win32.scd", "kind": "battle"},  # Fragments Of Sorrow
    # PP
    {"name": "music129.dat\\music129.win32.scd", "kind": "field"},  # Captain Hook Pirate Ship
    {"name": "music146.dat\\music146.win32.scd", "kind": "battle"},  # Pirate Gigue
    {"name": "music130.dat\\music130.win32.scd", "kind": "field"},  # Never Land Sky
    # PN
    {"name": "music117.dat\\music117.win32.scd", "kind": "field"},  # A Very Small Wish
    {"name": "music121.dat\\music121.win32.scd", "kind": "battle"},  # Monstrous Monstro
    # AT
    {"name": "music140.dat\\music140.win32.scd", "kind": "field", "dmca": True},  # Under The Sea
    {"name": "music147.dat\\music147.win32.scd", "kind": "battle"},  # An Adventure In Atlantica
    # HAW
    {"name": "music141.dat\\music141.win32.scd", "kind": "field"},  # Winnie The Pooh
    {"name": "music158.bgm\\music158.win32.scd", "kind": "field"},  # Bounce-O-Rama
    # Boss/Special
    {"name": "music099.bgm\\music099.win32.scd", "kind": "boss"},  # Dissapeared
    {"name": "music111.bgm\\music111.win32.scd", "kind": "boss"},  # Shrouding Dark Cloud
    {"name": "music131.dat\\music131.win32.scd", "kind": "boss"},  # Night Of Fate
    {"name": "music145.bgm\\music145.win32.scd", "kind": "boss"},  # Destiny Force
    {"name": "music150.bgm\\music150.win32.scd", "kind": "boss"},  # Squirming Evil
    {"name": "music151.bgm\\music151.win32.scd", "kind": "boss"},  # The Deep End
    {"name": "music156.bgm\\music156.win32.scd", "kind": "boss"},  # Guardando Nel Buio
    {"name": "music173.bgm\\music173.win32.scd", "kind": "boss"},  # No Time To Think
    {"name": "music174.bgm\\music174.win32.scd", "kind": "boss"},  # An Intense Situation
    {"name": "music185.bgm\\music185.win32.scd", "kind": "boss"},  # Forze Del Male
    {"name": "music196.bgm\\music196.win32.scd", "kind": "boss"},  # One Winged Angel
    {"name": "music197.bgm\\music197.win32.scd", "kind": "boss", "dmca": True},  # Night On Blad Mountain
    # Other
    {"name": "music110.bgm\\music110.win32.scd", "kind": "title"},  # Dearly Beloved
    {"name": "music193.bgm\\music193.win32.scd", "kind": "title"},  # Dearly Beloved Reprise
    {"name": "music122.dat\\music122.win32.scd", "kind": "battle"},  # Blast Away Gummi Ship I
    {"name": "music123.dat\\music123.win32.scd", "kind": "battle"},  # Blast Away Gummi Ship II
    {"name": "music124.dat\\music124.win32.scd", "kind": "field"},  # Shipmeisters Humoresque
    {"name": "music125.dat\\music125.win32.scd", "kind": "field"},  # Precious Stars In The Sky
    {"name": "music148.dat\\music148.win32.scd", "kind": "battle"},  # Blast Away Gummi Ship III
    {"name": "music114.bgm\\music114.win32.scd", "kind": "cutscene"},  # Unknown 1 [Heartless Has Come, Pt.3]
    {"name": "music142.bgm\\music142.win32.scd", "kind": "cutscene"},  # Dive Into The Heart (Part 9)
    {"name": "music143.bgm\\music143.win32.scd", "kind": "cutscene", "dmca": True},  # Mickey Mouse Club March
    {"name": "music160.bgm\\music160.win32.scd", "kind": "cutscene"},  # Kairi I
    {"name": "music161.bgm\\music161.win32.scd", "kind": "cutscene"},  # Kairi II
    {"name": "music163.bgm\\music163.win32.scd", "kind": "cutscene"},  # Villains Of A Sort
    {"name": "music170.bgm\\music170.win32.scd", "kind": "cutscene"},  # A Piece Of Peace
    {"name": "music172.bgm\\music172.win32.scd", "kind": "cutscene"},  # A Walk In Adante
    {"name": "music176.bgm\\music176.win32.scd", "kind": "cutscene"},  # Friends In My Heart
    {"name": "music178.bgm\\music178.win32.scd", "kind": "cutscene"},  # Treasured Memories
    {"name": "music180.bgm\\music180.win32.scd", "kind": "cutscene"},  # Kairi III
    {"name": "music187.bgm\\music187.win32.scd", "kind": "cutscene"},  # Just An Itty Bitty Too Much
    {"name": "music188.bgm\\music188.win32.scd", "kind": "cutscene"}  # It Began With A Letter
    # a few of the no intro duplicates removed along with BGMs that are short and don't loop.
    # {"name": "music194.bgm\\music194.win32.scd", "kind": "battle"}, #Guardando Nel Buio (No Intro)
    # {"name": "music097.bgm\\music097.win32.scd", "kind": "battle"}, #Dissapeared (No Intro)
    # {"name": "music098.bgm\\music098.win32.scd", "kind": "battle"}, #One Winged Angel (No Intro)
    # {"name": "music106.bgm\\music106.win32.scd", "kind": "unknown"}, #Where Is This (Part 1) (no loop)
    # {"name": "music107.bgm\\music107.win32.scd", "kind": "unknown"}, #Where Is This (Part 2) (no loop)
    # {"name": "music108.bgm\\music108.win32.scd", "kind": "unknown"}, #Where Is This (Part 3b) (no loop)
    # {"name": "music109.bgm\\music109.win32.scd", "kind": "unknown"}, #Where Is This (Part 4) (no loop)
    # {"name": "music113.bgm\\music113.win32.scd", "kind": "unknown"}, #The Heartless Has Come [Heartless Has Come, Pt.1] (no loop)
    # {"name": "music115.bgm\\music115.win32.scd", "kind": "unknown"}, #Where Is This (Part 3a) [Heartless Has Come, Pt.2] (no loop)
    # {"name": "music126.bgm\\music126.win32.scd", "kind": "unknown"}, #Turning The Key (no loop)
    # {"name": "music132.bgm\\music132.win32.scd", "kind": "unknown"}, #Dive Into The Heart (Parts 1-8) (would this actually work? there are 8 tracks in 1 here)
    # {"name": "music159.bgm\\music159.win32.scd", "kind": "unknown"}, #Once Upon a Time (no loop)
    # {"name": "music162.bgm\\music162.win32.scd", "kind": "unknown"}, #Villains Of A Sort (Short)
    # {"name": "music166.bgm\\music166.win32.scd", "kind": "unknown"}, #Strange Whispers (Part 5) (no loop)
    # {"name": "music167.bgm\\music167.win32.scd", "kind": "unknown"}, #Strange Whispers (Part 1) (no loop)
    # {"name": "music168.bgm\\music168.win32.scd", "kind": "unknown"}, #Strange Whispers
    # {"name": "music169.bgm\\music169.win32.scd", "kind": "unknown"}, #Strange Whispers (Part 3) (no loop)
    # {"name": "music171.bgm\\music171.win32.scd", "kind": "unknown"}, #Strange Whispers (Part 4) (no loop)
    # {"name": "music175.bgm\\music175.win32.scd", "kind": "unknown"}, #Unknown 3 [Kairi In Cabin] (no loop)
    # {"name": "music177.bgm\\music177.win32.scd", "kind": "unknown"}, #Tricksy Clock (no loop)
    # {"name": "music179.bgm\\music179.win32.scd", "kind": "unknown"}, #Miracle (no loop)
    # {"name": "music181.bgm\\music181.win32.scd", "kind": "unknown"}, #Hikari [Sora's Sacrifice] (no loop)
    # {"name": "music182.bgm\\music182.win32.scd", "kind": "unknown"}, #Mickey Mouse Club March (Intro) [Queen's Castle Drumroll] (no loop)
    # {"name": "music183.bgm\\music183.win32.scd", "kind": "unknown"}, #Oopsy-Daisy (no loop)
    # {"name": "music186.bgm\\music186.win32.scd", "kind": "unknown"}, #Unknown 2 [Plenty Of Hunny!] (no loop)
    # {"name": "music189.bgm\\music189.win32.scd", "kind": "unknown"}, #Blast Away Gummi Ship I (Sting 2) (no loop)
    # {"name": "music190.bgm\\music190.win32.scd", "kind": "unknown"}, #Blast Away Gummi Ship I (Sting) (no loop)
    # {"name": "music191.bgm\\music191.win32.scd", "kind": "unknown"}, #Beyond The Door (no loop)
    # {"name": "music192.bgm\\music192.win32.scd", "kind": "unknown"}, #Always On My Mind (Final Mix Version) (no loop)
    # {"name": "music100.win32.scd", "kind": "unknown"}, #empty 1kb file
    # {"name": "music137.win32.scd", "kind": "unknown"}, #empty 1kb file
]

recom_music_list = [
    {"name": '01F_Town_B.win32.scd', "kind": "battle"},  # Hand in Hand
    {"name": '01F_Town_F.win32.scd', "kind": "field"},  # Traverse Town
    {"name": '03F_Hercul.win32.scd', "kind": "field"},  # Olympus Coliseum
    {"name": '03F_Hercules_B.win32.scd', "kind": "battle"},  # Go for It
    {"name": '17_Halloween_F.win32.scd', "kind": "field", "dmca": True},  # This is Halloween
    {"name": '18_Helloween_B.win32.scd', "kind": "battle"},  # Spooks of Halloween Town
    {"name": '19_Alice_Field.win32.scd', "kind": "field"},  # Welcome to Wonderland
    {"name": '20_Event_13thFloor.win32.scd', "kind": "field"},  # The 13th Floor
    {"name": '21_Alice_Battle.win32.scd', "kind": "battle"},  # To Our Surprise
    {"name": '22_PeterPan_Field.win32.scd', "kind": "field"},  # Captain Hook's Pirate Ship
    {"name": '23_PeterPan_Battle.win32.scd', "kind": "battle"},  # Pirate's Gigue
    {"name": '24_Pinocchio_Field.win32.scd', "kind": "field"},  # A Very Small Wish
    {"name": '25_Aladdin_F.win32.scd', "kind": "field"},  # A Day in Agrabah
    {"name": '26_Aladdin_B.win32.scd', "kind": "battle"},  # Arabian Dream
    {"name": '27_Pinocchio_B.win32.scd', "kind": "battle"},  # Monstrous Monstro
    {"name": '28_Forget_F.win32.scd', "kind": "field"},  # Castle Oblivion
    {"name": '29_Forget_B.win32.scd', "kind": "battle"},  # Forgotten Challenge
    {"name": '30_Twilight_F.win32.scd', "kind": "field"},  # Lazy Afternoons
    {"name": '31_Twilight_B.win32.scd', "kind": "battle"},  # Sinister Sundown
    {"name": '32_Destiny_F.win32.scd', "kind": "field"},  # Destiny Islands (Re:Chain of Memories Version)
    {"name": '33_Destiny_B.win32.scd', "kind": "battle"},  # Night of Fate (Re:Chain of Memories Version)
    {"name": '36_WinnieThePooh.win32.scd', "kind": "field"},  # Winnie the Pooh
    {"name": '38_UnderTheSea.win32.scd', "kind": "field", "dmca": True},  # Under the Sea
    {"name": '39_LittleMermaid_B.win32.scd', "kind": "battle"},  # An Adventure in Atlantica
    {"name": '42_Hollow_F.win32.scd', "kind": "field"},  # Hollow Bastion
    {"name": '43_Hollow_B.win32.scd', "kind": "battle"},  # Scherzo Di Notte
    {"name": '44_PooGame1.win32.scd', "kind": "field"},  # March-A-Long
    {"name": '45_PoohGame2.win32.scd', "kind": "battle"},  # Dash-A-Long
    # boss/special
    {"name": '16_XIIIBoss1.win32.scd', "kind": "boss"},  # The 13th Struggle (Re:Chain of Memories Version)
    {"name": '34_Boss_RikuAnsem.win32.scd', "kind": "boss"},  # Revenge of Chaos
    {"name": '35_Boss_NiseRiku.win32.scd', "kind": "boss"},  # The Force in You
    {"name": '37_LastBoss1.win32.scd', "kind": "boss"},  # Graceful Assassin
    {"name": '40_LastBoss2.win32.scd', "kind": "boss"},  # Scythe of Petals
    {"name": '41_LastBoss3.win32.scd', "kind": "boss"},  # Lord of the Castle
    {"name": 'Boss1.win32.scd', "kind": "boss"},  # Struggle Away
    {"name": 'Boss2_World.win32.scd', "kind": "boss"},  # The Fight for My Friends
    {"name": 'Event1_Kinpak.win32.scd', "kind": "boss"},  # Face It!
    # Other
    {"name": 'Title.win32.scd', "kind": "title"},  # Dearly Beloved (Re:Chain of Memories Version)
    {"name": '12_Event_Yuttari.win32.scd', "kind": "cutscene"},  # La Pace
    {"name": '13_Event_Namine.win32.scd', "kind": "cutscene"},  # Naminé
    {"name": '14_Event_Odayaka.win32.scd', "kind": "cutscene"},  # Piccolo Resto
    {"name": '15_Event_XIII.win32.scd', "kind": "cutscene"},  # Thirteenth Discretion
    {"name": 'Event_Unrest.win32.scd', "kind": "cutscene"},  # Disquieting
    {"name": 'Event2_.win32.scd', "kind": "cutscene"},  # Just Wondering
    {"name": 'Event4.win32.scd', "kind": "cutscene"}  # Scent of Silence
]

bbs_music_list = [
    {"name": "001sinde_f.win32.scd", "kind": "field", "dmca": True},  # Bibbidi-Bobbidi-Boo (Castle of Dreams Field)
    {"name": "002sinde_b.win32.scd", "kind": "battle"},  # Castle Escapade (Castle of Dreams Battle Music)
    {"name": "003nemure_f.win32.scd", "kind": "field"},  # The Silent Forest (Enchanted Dominion Field Music)
    {"name": "004nemure_b.win32.scd", "kind": "battle"},  # The Rustling Forest (Enchanted Dominion Battle Music)
    {"name": "005syugyo_f.win32.scd", "kind": "field"},  # The Promised Beginning (The Land of Departure Field Music)
    {"name": "006syugyo_b.win32.scd", "kind": "battle"},  # Future Masters (The Land of Departure Battle Music)
    {"name": "007shira_f.win32.scd", "kind": "field"},  # The Secret Whispers (Dwarf Woodlands Field Music)
    {"name": "008shira_b.win32.scd", "kind": "battle"},  # Risky Romp (Dwarf Woodlands Battle Music)
    {"name": "009raydi_f.win32.scd", "kind": "field"},  # Radiant Garden (Radiant Garden Field Music)
    {"name": "010raydi_b.win32.scd", "kind": "battle"},  # Black Garden (Radiant Garden Battle Music)
    {"name": "011distow_f.win32.scd", "kind": "field", "dmca": True},  # Mickey Mouse March (Disney Town Field Music)
    {"name": "012distow_b.win32.scd", "kind": "battle"},  # Up Down Adventure (Disney Town Battle Music)
    {"name": "013never_f.win32.scd", "kind": "field"},  # Daydream upon Neverland (Neverland Field Music)
    {"name": "014never_b.win32.scd", "kind": "battle"},  # Neverland's Scherzo (Neverland Battle Music)
    {"name": "015herc_f.win32.scd", "kind": "field"},  # Olympus Coliseum (Olympus Coliseum Field Music)
    {"name": "016herc_b.win32.scd", "kind": "battle"},  # Go for It! (Olympus Coliseum Battle Music)
    {"name": "017riro_f.win32.scd", "kind": "field"},  # Hau'oli, Hau'oli (Deep Space Field Music)
    {"name": "018riro_b.win32.scd", "kind": "battle"},  # Mákaukau? (Deep Space Battle Music)
    {"name": "019iensid_f.win32.scd", "kind": "field"},  # Magical Mystery (Mysterious Tower Field Music)
    {"name": "020iensid_b.win32.scd", "kind": "battle"},  # Working Together (Mysterious Tower Battle Music)
    {"name": "021kouya_f.win32.scd", "kind": "field"},  # Keyblade Graveyard Horizon
    {"name": "022disice_f.win32.scd", "kind": "field"},  # Destiny Islands (Birth by Sleep Version)
    {"name": "023tsusin_f.win32.scd", "kind": "field"},  # Dearly Dreams
    {"name": "030youki.win32.scd", "kind": "cutscene"},  # Innocent Times
    {"name": "031isamashi.win32.scd", "kind": "cutscene"},  # Cheers for the Brave
    {"name": "032odayaka.win32.scd", "kind": "cutscene"},  # Peaceful Hearts
    {"name": "033fuon.win32.scd", "kind": "cutscene"},  # Drops of Poison
    {"name": "034kanasii.win32.scd", "kind": "cutscene"},  # Tears of the Light
    {"name": "035kinpaku.win32.scd", "kind": "cutscene"},  # Shaded Truths
    {"name": "036seisin.win32.scd", "kind": "field"},  # Dive into the Heart -Destati-
    {"name": "037yami.win32.scd", "kind": "cutscene"},  # Beyond the Door
    {"name": "038ria_deai.win32.scd", "kind": "cutscene"},  # Eternal Moments
    {"name": "050title.win32.scd", "kind": "title"},  # Dearly Beloved (Birth by Sleep Version)
    {"name": "051worldmap.win32.scd", "kind": "field"},  # The Worlds
    {"name": "060tera.win32.scd", "kind": "battle"},  # Terra
    {"name": "061aqua.win32.scd", "kind": "cutscene"},  # Aqua
    {"name": "062ven.win32.scd", "kind": "cutscene"},  # Ventus
    {"name": "063kairi1.win32.scd", "kind": "cutscene"},  # Kairi (Birth by Sleep Version)
    {"name": "066peet.win32.scd", "kind": "cutscene"},  # Hero or Heel
    {"name": "067disvill.win32.scd", "kind": "cutscene"},  # Villains of a Sort
    {"name": "068zea.win32.scd", "kind": "cutscene"},  # Xehanort
    {"name": "069braig.win32.scd", "kind": "cutscene"},  # Organization XIII
    {"name": "070key_l.win32.scd", "kind": "cutscene"},  # The Key of Light
    {"name": "071key_d.win32.scd", "kind": "cutscene"},  # The Key of Darkness
    {"name": "072key_l_d.win32.scd", "kind": "battle"},  # The Key
    {"name": "073kizuna.win32.scd", "kind": "cutscene"},  # Destiny's Union
    {"name": "074zack.win32.scd", "kind": "cutscene"},  # A Date with Fate
    {"name": "100ice1_128.win32.scd", "kind": "field"},  # It's a Small World
    {"name": "101ice1_132.win32.scd", "kind": "field"},  # It's a Small World
    {"name": "102ice_2.win32.scd", "kind": "field"},  # Dessert Paradise
    {"name": "103fruit.win32.scd", "kind": "field"},  # Fresh Fruit Balls
    {"name": "104dice.win32.scd", "kind": "field"},  # Dice & Shine
    {"name": "105poomini.win32.scd", "kind": "field"},  # March-A-Long
    {"name": "106cartrace.win32.scd", "kind": "battle"},  # Go! Go! Rumble Racer
    {"name": "107syugyo.win32.scd", "kind": "field"},
    # The Promised Beginning (The Land of Departure Field Music) (Command Board)
    {"name": "108riro.win32.scd", "kind": "field"},  # Hau'oli, Hau'oli (Deep Space Field Music) (Command Board)
    {"name": "109training.win32.scd", "kind": "battle"},  # Road to a Hero (Birth by Sleep Version)
    {"name": "113kh1tit.win32.scd", "kind": "title"},  # Dearly Beloved (From KH1, used at Battle Report)
    {"name": "114raceview.win32.scd", "kind": "field"},  # Big Race
    # {"name": "116icon.win32.scd", "kind": "field"}, #Dearly Beloved (Birth by Sleep Version) (10-Second Version)
    {"name": "117short_l2.win32.scd", "kind": "field"},  # Another Side - Battle Ver. - (Birth by Sleep Version)
    {"name": "118gumi.win32.scd", "kind": "field"},  # Blast Away! -Gummi Ship II- (From Original KH1)
    {"name": "119desti.win32.scd", "kind": "field"},  # Destiny Islands (From Original KH1)
    {"name": "120hand.win32.scd", "kind": "battle"},  # Hand in Hand (From Original KH1)
    # {"name": "123rev.win32.scd", "kind": "field"}, #Mysterious Figure's Time Reverse Loop
    {"name": "124dp_amb.win32.scd", "kind": "field"},  # Ambient Wind Sounds from Destroyed Land of Departure
    {"name": "125yami_f.win32.scd", "kind": "field"},  # Night of the Dark Dream (Realm of Darkness Field Music)
    {"name": "126yami_b.win32.scd", "kind": "battle"},  # Night of Tragedy (Realm of Darkness Battle Music)
    # Boss/Special
    {"name": "040anba_b1.win32.scd", "kind": "boss"},  # The Tumbling
    {"name": "041anba_b2.win32.scd", "kind": "boss"},  # Unforgettable
    {"name": "042dis_b1.win32.scd", "kind": "boss"},  # Vim and Vigor -Birth by Sleep Version-
    {"name": "043dis_b2.win32.scd", "kind": "boss"},  # The Encounter -Birth by Sleep Version-
    {"name": "044vanita_b.win32.scd", "kind": "boss"},  # Enter the Darkness
    {"name": "045anthem_b.win32.scd", "kind": "boss"},  # Black Powder
    {"name": "046last_b1.win32.scd", "kind": "boss"},  # Unbreakable Chains
    {"name": "047last_b2.win32.scd", "kind": "boss"},  # Dismiss
    {"name": "048hanyo_b1.win32.scd", "kind": "boss"},  # Extreme Encounters
    {"name": "049hanyo_b2.win32.scd", "kind": "boss"},  # Destiny's Force
    {"name": "110han_bt1.win32.scd", "kind": "boss"},  # Rowdy Rumble -Birth by Sleep Version-
    {"name": "111han_bt2.win32.scd", "kind": "boss"},  # Shrouding Dark Cloud -Birth by Sleep Version-
    {"name": "112rage_bt.win32.scd", "kind": "boss"},  # Rage Awakened -The Origin-
    {"name": "115boss.win32.scd", "kind": "boss"},  # Enter the Void
    {"name": "122nazono.win32.scd", "kind": "boss"},  # Dark Impetus
    {"name": "127Xeha_b.win32.scd", "kind": "boss"},  # Forze dell Oscurita
    {"name": "128Eraqu_b.win32.scd", "kind": "boss"},  # Master, Tell Me the Truth
    {"name": "129Pure_b.win32.scd", "kind": "boss"},  # Hunter of the Dark
    {"name": "130Mons_b.win32.scd", "kind": "boss"}  # Monstrous Monstro -Arena Ver.-
]

ddd_music_list = [
    {"name": "bgm_001.win32.scd", "kind": "field"},  # Traverse in Trance (Traverse Town Field Music)
    {"name": "bgm_002.win32.scd", "kind": "field"},  # One for All (Country of the Musketeers Field Music)
    {"name": "bgm_003.win32.scd", "kind": "field"},  # The Fun Fair (Prankster's Paradise Field Music)
    {"name": "bgm_004.win32.scd", "kind": "field"},  # A Very Small Wish (Monstro Field Music)
    {"name": "bgm_005.win32.scd", "kind": "field", "dmca": True},  # Symphony No. 6 Pastoral Op. 68 (Sora's SoS Music)
    {"name": "bgm_006.win32.scd", "kind": "field", "dmca": True},  # The Nutcracker Suite Op. 71 (Riku's SoS Music)
    {"name": "bgm_007.win32.scd", "kind": "field", "dmca": True},  # L'Apprenti Sorcier (Symphony of Sorcery Music)
    {"name": "bgm_008.win32.scd", "kind": "battle", "dmca": True},  # Night on Bald Mountain
    {"name": "bgm_009.win32.scd", "kind": "field"},  # La Cloche (La Cité des Cloches Field Music)
    {"name": "bgm_010.win32.scd", "kind": "field"},  # Magical Mystery (Mysterious Tower Field Music)
    {"name": "bgm_011.win32.scd", "kind": "field"},  # Access the Grid (The Grid Field Music)
    {"name": "bgm_012.win32.scd", "kind": "field"},  # Sacred Distance (KH3D The World That Never Was Field Music)
    {"name": "bgm_013.win32.scd", "kind": "field"},  # Sacred Moon (KH2 The World That Never Was Field Music)
    {"name": "bgm_014.win32.scd", "kind": "battle"},  # Hand to Hand (Traverse Town Battle Music)
    {"name": "bgm_015.win32.scd", "kind": "battle"},  # All for One (Country of the Musketeers Battle Music)
    {"name": "bgm_016.win32.scd", "kind": "battle"},  # Prankster's Party (Prankster's Paradise Battle Music)
    {"name": "bgm_017.win32.scd", "kind": "battle"},  # Monstrous Monstro (Monstro Battle Music)
    {"name": "bgm_018.win32.scd", "kind": "battle"},  # Le Sanctuaire (La Cité des Cloches Field Music)
    {"name": "bgm_019.win32.scd", "kind": "battle"},  # Working Together (Mysterious Tower Field Music)
    {"name": "bgm_020.win32.scd", "kind": "battle"},  # Digital Domination (The Grid Battle Music)
    {"name": "bgm_021.win32.scd", "kind": "battle"},  # Deep Drop (KH3D The World That Never Was Battle Music)
    {"name": "bgm_022.win32.scd", "kind": "battle"},  # Deep Drive (KH2 The World That Never Was Battle Music)
    {"name": "bgm_035.win32.scd", "kind": "field"},  # Dream Matchup
    {"name": "bgm_036.win32.scd", "kind": "field"},  # The Flick Finalist
    {"name": "bgm_037.win32.scd", "kind": "field"},  # The World of Dream Drops (World Map Music)
    {"name": "bgm_038.win32.scd", "kind": "title"},  # Dearly Beloved (Dream Drop Distance Version)
    {"name": "bgm_039.win32.scd", "kind": "cutscene"},  # Innocent Times
    {"name": "bgm_040.win32.scd", "kind": "cutscene"},  # Cheers for the Brave
    {"name": "bgm_041.win32.scd", "kind": "cutscene"},  # Peaceful Hearts
    {"name": "bgm_042.win32.scd", "kind": "cutscene"},  # Drops of Poison
    {"name": "bgm_043.win32.scd", "kind": "cutscene"},  # Tears of the Light
    {"name": "bgm_044.win32.scd", "kind": "cutscene"},  # Shaded Truths
    {"name": "bgm_045.win32.scd", "kind": "cutscene"},  # Broken Reality
    {"name": "bgm_046.win32.scd", "kind": "cutscene"},  # Ever After
    {"name": "bgm_047.win32.scd", "kind": "cutscene"},  # Sora
    {"name": "bgm_048.win32.scd", "kind": "cutscene"},  # Riku
    {"name": "bgm_050.win32.scd", "kind": "battle"},  # Road to a Hero
    {"name": "bgm_051.win32.scd", "kind": "cutscene"},  # Villains of a Sort
    {"name": "bgm_052.win32.scd", "kind": "cutscene"},  # Strange Whispers
    {"name": "bgm_053.win32.scd", "kind": "battle"},  # Night of Fate
    {"name": "bgm_059.win32.scd", "kind": "cutscene"},  # The Key of Light
    {"name": "bgm_060.win32.scd", "kind": "cutscene"},  # The Key of Darkness
    {"name": "bgm_061.win32.scd", "kind": "cutscene"},  # Xehanort
    {"name": "bgm_062.win32.scd", "kind": "cutscene"},  # Xehanort -The Early Years-
    {"name": "bgm_063.win32.scd", "kind": "cutscene"},  # Xigbar
    {"name": "bgm_065.win32.scd", "kind": "cutscene"},  # Organization XIII
    {"name": "bgm_066.win32.scd", "kind": "cutscene"},  # Destiny's Union
    {"name": "bgm_067.win32.scd", "kind": "field"},  # Dream Eaters
    {"name": "bgm_068.win32.scd", "kind": "cutscene"},  # TWISTER -KINGDOM MIX-
    {"name": "bgm_069.win32.scd", "kind": "cutscene"},  # CALLING -KINGDOM MIX- (No Intro)
    {"name": "bgm_070.win32.scd", "kind": "title"},  # Dearly Beloved (End of Symphony of Sorcery Cutscene 1)
    {"name": "bgm_071.win32.scd", "kind": "title"},  # Dearly Beloved (End of Symphony of Sorcery Cutscene 2)
    {"name": "bgm_072.win32.scd", "kind": "field", "dmca": True},  # Symphony of Sorcery Music
    {"name": "bgm_073.win32.scd", "kind": "field", "dmca": True},  # Symphony of Sorcery Music (Spellican Battle Music)
    {"name": "bgm_074.win32.scd", "kind": "field", "dmca": True},  # Symphony of Sorcery Music
    {"name": "bgm_075.win32.scd", "kind": "cutscene"},  # The Dream
    {"name": "bgm_076.win32.scd", "kind": "cutscene"},  # The Nightmare
    {"name": "bgm_077.win32.scd", "kind": "cutscene"},  # Link to All
    {"name": "bgm_078.win32.scd", "kind": "cutscene"},  # Distant from You...
    {"name": "bgm_083.win32.scd", "kind": "field"},  # Sweet Spirits
    {"name": "bgm_084.win32.scd", "kind": "field"},  # Ready to Rush
    {"name": "bgm_085.win32.scd", "kind": "field"},  # Victor's Pride
    {"name": "bgm_086.win32.scd", "kind": "battle"},  # Keyblade Cycle
    {"name": "bgm_089.win32.scd", "kind": "field"},  # My Heart's Descent
    # {"name": "bgm_091.win32.scd", "kind": "battle"}, # Dream Drop Distance -The Next Awakening- (no loop)
    {"name": "bgm_095.win32.scd", "kind": "battle"},  # SOMEDAY -KINGDOM MIX-
    # {"name": "bgm_096.win32.scd", "kind": "unknown"}, # Hand in Hand (KH2 Ending Version)
    # {"name": "bgm_097.win32.scd", "kind": "unknown"}, # Oopsy-Daisy
    {"name": "bgm_098.win32.scd", "kind": "cutscene"},  # TWISTER -KINGDOM MIX-
    {"name": "bgm_099.win32.scd", "kind": "cutscene"},  # CALLING -KINGDOM MIX- (With Intro)
    # {"name": "bgm_112.win32.scd", "kind": "title"}, # Dearly Beloved (Kingdom Hearts II Version)
    # {"name": "bgm_113.win32.scd", "kind": "title"}, # Dearly Beloved -Reprise-
    {"name": "bgm_114.win32.scd", "kind": "cutscene"},  # Roxas
    {"name": "bgm_115.win32.scd", "kind": "cutscene"},  # Ventus
    # boss/Special
    {"name": "bgm_023.win32.scd", "kind": "boss"},  # Majestic Wings
    {"name": "bgm_024.win32.scd", "kind": "boss"},  # UNTAMABLE
    {"name": "bgm_025.win32.scd", "kind": "boss"},  # Ice-hot Lobster
    {"name": "bgm_026.win32.scd", "kind": "boss"},  # Gigabyte Mantis
    {"name": "bgm_027.win32.scd", "kind": "boss"},  # Rowdy Rumble
    {"name": "bgm_028.win32.scd", "kind": "boss"},  # The Encounter -Birth by Sleep Version-
    {"name": "bgm_029.win32.scd", "kind": "boss"},  # L'Oscurità dell' Ignoto
    {"name": "bgm_030.win32.scd", "kind": "boss"},  # L'Eminenza Oscura I
    {"name": "bgm_031.win32.scd", "kind": "boss"},  # L'Eminenza Oscura II
    {"name": "bgm_032.win32.scd", "kind": "boss"},  # L'Impeto Oscuro
    {"name": "bgm_033.win32.scd", "kind": "boss"},  # Destiny's Force
    {"name": "bgm_034.win32.scd", "kind": "boss"},  # Shrouding Dark Cloud
    {"name": "bgm_087.win32.scd", "kind": "boss"},  # Storm Diver
    {"name": "bgm_088.win32.scd", "kind": "boss"},  # Wild Blue
    {"name": "bgm_093.win32.scd", "kind": "boss"},  # The Eye of Darkness
    {"name": "bgm_094.win32.scd", "kind": "boss"},  # The Dread of Night
    {"name": "bgm_116.win32.scd", "kind": "boss"}   # Rinzler Recompiled
]
